﻿using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TransportGuideContracts.BindingModels;
using TransportGuideContracts.BusinessLogicsContracts;
using TransportGuideContracts.SearchModels;
using TransportGuideContracts.StoragesContracts;
using TransportGuideContracts.ViewModels;

namespace BusinessLogic.BusinessLogic
{
	public class RouteLogic : IRouteLogic
	{
		private readonly ILogger _logger;
		private readonly IRouteStorage _routeStorage;

		public RouteLogic(ILogger<RouteLogic> logger, IRouteStorage routeStorage)
		{
			_logger = logger;
			_routeStorage = routeStorage;
		}

		public bool Create(RouteBindingModel model)
		{
			CheckModel(model);
			if (_routeStorage.Insert(model) == null)
			{
				_logger.LogWarning("Insert operation failed");
				return false;
			}
			return true;
		}

		public bool Delete(RouteBindingModel model)
		{
			CheckModel(model, false);
			_logger.LogInformation("Delete. Id:{Id}", model.Id);
			if (_routeStorage.Delete(model) == null)
			{
				_logger.LogWarning("Delete operation failed");
				return false;
			}
			return true;
		}

		public RouteViewModel? ReadElement(RouteSearchModel model)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			_logger.LogInformation("ReadElement. RouteName:{RouteName}.Id:{ Id}", model.Name, model.Id);
			var element = _routeStorage.GetElement(model);
			if (element == null)
			{
				_logger.LogWarning("ReadElement element not found");
				return null;
			}
			_logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
			return element;
		}

		public List<RouteViewModel>? ReadList(RouteSearchModel? model)
		{
			_logger.LogInformation("ReadList. RouteName:{RoutetName}.Id:{ Id}", model?.Name, model?.Id);
			var list = model == null ? _routeStorage.GetFullList() : _routeStorage.GetFilteredList(model);
			if (list == null)
			{
				_logger.LogWarning("ReadList return null list");
				return null;
			}
			_logger.LogInformation("ReadList. Count:{Count}", list.Count);
			return list;
		}

		public string TestInsertList(int num)
		{
			return _routeStorage.TestInsertList(num);
		}

		public string TestReadList(int num)
		{
			return _routeStorage.TestReadList(num);
		}

		public bool Update(RouteBindingModel model)
		{
			CheckModel(model);
			if (_routeStorage.Update(model) == null)
			{
				_logger.LogWarning("Update operation failed");
				return false;
			}
			return true;
		}
		private void CheckModel(RouteBindingModel model, bool withParams =
	 true)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (!withParams)
			{
				return;
			}
			if (string.IsNullOrEmpty(model.Name))
			{
				throw new ArgumentNullException("Нет названия",nameof(model.Name));
			}
			if (string.IsNullOrEmpty(model.IP))
			{
				throw new ArgumentNullException("No IP", nameof(model.IP));
			}
			_logger.LogInformation("Route. Name:{Name}. IP:{ IP}. Id: { Id}", model.Name, model.IP, model.Id);
			var element = _routeStorage.GetElement(new RouteSearchModel
			{
				Name = model.Name
			});
			if (element != null && element.Id != model.Id)
			{
				throw new InvalidOperationException("Route с таким названием уже есть");
			}
		}
	}
}
